<?xml version='1.0' encoding='utf-8'?>
<xsl:stylesheet version='2.0'
     xmlns:xsl='http://www.w3.org/1999/XSL/Transform'
     xmlns:xs='http://www.w3.org/2001/XMLSchema'
     xmlns:zw='http://icebearsoft.euweb.cz'
     exclude-result-prefixes='#all'>

<!-- Delimiter between the key and page numbers -->
<xsl:param name='delim' as='xs:string'>\pfill </xsl:param>

<!-- For headers -->
<xsl:param name='lethead-prefix' as='xs:string'>\header{</xsl:param>
<xsl:param name='lethead-suffix' as='xs:string'>}</xsl:param>
<xsl:param name='symbols' as='xs:string'>Symbols</xsl:param>

<!-- Is CH a separate character? -->
<xsl:variable name='use-ch' as='xs:boolean' select="$lang = 'cs' or $lang = 'sk'"/>

<xsl:output method='text' indent='no'/>

<!-- Sorted items -->
<xsl:variable name='sorted-list' as='element()'>
  <xsl:call-template name='sorted-index'/>
</xsl:variable>

<xsl:template name='noheaders'>
  <xsl:param name='sorted-items' as='element()' select='$sorted-list'/>
  <xsl:text>\begin{theindex}</xsl:text>
  <xsl:apply-templates select='$sorted-items'/>
  <xsl:text>&#10;\end{theindex}&#10;</xsl:text>
</xsl:template>

<xsl:template name='headers'>
  <xsl:param name='sorted-items' as='element()' select='$sorted-list'/>
  <xsl:text>\begin{theindex}</xsl:text>
  <xsl:for-each-group select='$sorted-items/*'
       group-adjacent="if (matches(@id, '^[^\p{L}]')) then $symbols else
                       if ($use-ch and upper-case(substring(@id, 1, 2)) = 'CH') then 'CH'
                       else zw:key(upper-case(substring(@id, 1, 1)))">
    <xsl:value-of select="concat('&#10;&#10;',
        $lethead-prefix, current-grouping-key(), $lethead-suffix)"/>
    <xsl:apply-templates select='current-group()'/>
  </xsl:for-each-group>
  <xsl:text>&#10;\end{theindex}&#10;</xsl:text>
</xsl:template>

<xsl:function name='zw:key'>
  <xsl:param name='key' as='xs:string'/>
  <xsl:value-of select="if ($lang != 'cs' and $lang != 'sk') then $key
       else translate($key, 'ÁÄĎÉĚÍĹĽŇÓÔÖŤÚŮÜÝ', 'AADEEILLNOOOTUUUY')"/>
</xsl:function>

<xsl:template match='sorted-index'>
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match='*'>
  <xsl:param name='item' as='xs:string'>item </xsl:param>
  <xsl:value-of select="concat('&#10;\', $item, @text)"/>
  <xsl:if test='page'>
    <xsl:value-of select='$delim'/>
    <xsl:value-of separator=', '
         select="for $page in (page) return concat(
           if ($page/@style = $default-page-style)
              then '' else concat('\', $page/@style, '{'),
           if ($page/@rangeTo)
              then concat('\hbox{', $page/@number, '--', $page/@rangeTo, '}')
              else $page/@number,
           if ($page/@style = $default-page-style) then '' else '}')"/>
  </xsl:if>
  <xsl:apply-templates select='* except page'>
    <xsl:with-param name='item'>
      <xsl:value-of select='concat("sub", $item)'/>
    </xsl:with-param>
  </xsl:apply-templates>
</xsl:template>

</xsl:stylesheet>
