<?xml version='1.0' encoding='utf-8'?>
<xsl:stylesheet version='2.0'
     xmlns:xsl='http://www.w3.org/1999/XSL/Transform'
     xmlns:xs='http://www.w3.org/2001/XMLSchema'
     exclude-result-prefixes='#all'>

<!-- The input file and its encoding -->
<xsl:param name='index' as='xs:string' required='yes'/>
<xsl:param name='enc' as='xs:NMTOKEN'>utf-8</xsl:param>
<xsl:param name='file-separator' as='xs:string'>,</xsl:param>

<!-- Index file specifiers used inside regex (specials must be escaped) -->
<xsl:param name='encap' as='xs:string'>\|</xsl:param>
<xsl:param name='actual' as='xs:string'>@</xsl:param>
<xsl:param name='escape' as='xs:string'>\\</xsl:param>
<xsl:param name='level' as='xs:string'>!</xsl:param>
<xsl:param name='quote' as='xs:string'>"</xsl:param>

<!-- Index file specifiers used as normal strings -->
<xsl:param name='range-open' as='xs:string'>(</xsl:param>
<xsl:param name='range-close' as='xs:string'>)</xsl:param>

<!-- Used if no style given in the \index command -->
<xsl:variable name='default-page-style' as='xs:string'>#default-style</xsl:variable>

<!-- Parse the index file(s) into a series of elements which may be nested -->
<xsl:variable name='index-items' as='item()*'>
  <xsl:analyze-string
       select='string-join(for $fn in (tokenize($index, $file-separator))
                           return unparsed-text($fn, $enc), "")'
       regex='\\(\S+?)\s*\{{(.+?)\}}\{{(-?\d+?)\}}\r?\n' flags='s'>
    <xsl:matching-substring>
      <xsl:variable name='entry' as='xs:string' select='regex-group(1)'/>
      <xsl:variable name='page' as='xs:integer' select='xs:integer(regex-group(3))'/>
      <xsl:analyze-string select='normalize-space(regex-group(2))'
                          regex='(.+[^{$escape}{$quote}]){$encap}\s*(.+)'>
        <xsl:matching-substring>
          <xsl:call-template name='make-index-item'>
            <xsl:with-param name='entry' tunnel='yes' select='$entry'/>
            <xsl:with-param name='page' tunnel='yes' select='$page'/>
            <xsl:with-param name='style' tunnel='yes' select='regex-group(2)'/>
            <xsl:with-param name='text' select='regex-group(1)'/>
          </xsl:call-template>
        </xsl:matching-substring>
        <xsl:non-matching-substring>
          <xsl:call-template name='make-index-item'>
            <xsl:with-param name='entry' tunnel='yes' select='$entry'/>
            <xsl:with-param name='page' tunnel='yes' select='$page'/>
            <xsl:with-param name='text' select='.'/>
          </xsl:call-template>
        </xsl:non-matching-substring>
      </xsl:analyze-string>
    </xsl:matching-substring>
    <xsl:non-matching-substring>
      <xsl:message>
The input file contains illegal entry: <xsl:value-of select='normalize-space(.)'/>
      </xsl:message>
    </xsl:non-matching-substring>
  </xsl:analyze-string>
</xsl:variable>

<!-- Analyze the index item, invoke nesting by recursion if "level" required -->
<xsl:template name='make-index-item'>
  <xsl:param name='entry' as='xs:string' required='yes' tunnel='yes'/>
  <xsl:param name='page' as='xs:integer' required='yes' tunnel='yes'/>
  <xsl:param name='style' as='xs:string' tunnel='yes' select='$default-page-style'/>
  <xsl:param name='text' as='xs:string' required='yes'/>
  <xsl:element name='{$entry}'>
    <xsl:analyze-string select='$text'
                        regex='(.+?[^{$escape}{$quote}]){$level}\s*(.+)'>
      <xsl:matching-substring>
        <xsl:call-template name='key-text'>
          <xsl:with-param name='text' select='regex-group(1)'/>
        </xsl:call-template>
        <xsl:call-template name='make-index-item'>
          <xsl:with-param name='text' select='regex-group(2)'/>
        </xsl:call-template>
      </xsl:matching-substring>
      <xsl:non-matching-substring>
        <xsl:call-template name='key-text'>
          <xsl:with-param name='text' select='.'/>
        </xsl:call-template>
        <page number='{$page}' style='{if ($style) then $style else $default-page-style}'>
          <xsl:choose>
            <xsl:when test='starts-with($style, $range-open)'>
              <xsl:attribute name='style'>
                <xsl:value-of
                     select='if ($style = $range-open)
                     then $default-page-style else substring-after($style, $range-open)'/>
              </xsl:attribute>
              <xsl:attribute name='range'>open</xsl:attribute>
            </xsl:when>
            <xsl:when test='$style = $range-close'>
              <xsl:attribute name='style'>
                <xsl:value-of select='$default-page-style'/>
              </xsl:attribute>
              <xsl:attribute name='range'>close</xsl:attribute>
            </xsl:when>
          </xsl:choose>
        </page>
      </xsl:non-matching-substring>
    </xsl:analyze-string>
  </xsl:element>
</xsl:template>

<!-- Split the item into a sort key and display text -->
<xsl:template name='key-text'>
  <xsl:param name='text' as='xs:string' required='yes'/>
  <xsl:analyze-string select='$text'
                      regex='(.+?[^{$escape}{$quote}]){$actual}\s*(.+)'>
    <xsl:matching-substring>
      <key>
        <xsl:call-template name='unquote-text'>
          <xsl:with-param name='text' select='regex-group(1)'/>
        </xsl:call-template>
      </key>
      <text>
        <xsl:call-template name='unquote-text'>
          <xsl:with-param name='text' select='regex-group(2)'/>
        </xsl:call-template>
      </text>
    </xsl:matching-substring>
    <xsl:non-matching-substring>
      <key>
        <xsl:call-template name='unquote-text'>
          <xsl:with-param name='text' select='.'/>
        </xsl:call-template>
      </key>
      <text>
        <xsl:call-template name='unquote-text'>
          <xsl:with-param name='text' select='.'/>
        </xsl:call-template>
      </text>
    </xsl:non-matching-substring>
  </xsl:analyze-string>
</xsl:template>

<!-- Remove quote characters preceding specials -->
<xsl:template name='unquote-text'>
  <xsl:param name='text' as='xs:string' required='yes'/>
  <xsl:analyze-string select='$text' regex='(^{$quote}|[^{$escape}]{$quote})(.)'>
    <xsl:matching-substring>
      <xsl:value-of select='substring(regex-group(1), 1, string-length(regex-group(1)) - 1)'/>
      <xsl:value-of select='regex-group(2)'/>
    </xsl:matching-substring>
    <xsl:non-matching-substring>
      <xsl:value-of select='.'/>
    </xsl:non-matching-substring>
  </xsl:analyze-string>
</xsl:template>

<!-- Debug index parser -->
<xsl:template name='parsed-index'>
  <parsed-index>
    <xsl:copy-of select='$index-items'/>
  </parsed-index>
</xsl:template>

</xsl:stylesheet>
