<?xml version='1.0' encoding='utf-8'?>
<xsl:stylesheet version='2.0'
     xmlns:xsl='http://www.w3.org/1999/XSL/Transform'
     xmlns:xs='http://www.w3.org/2001/XMLSchema'
     exclude-result-prefixes='#all'>

<!-- Sort options -->
<xsl:param name='case-order' as='xs:NMTOKEN'>lower-first</xsl:param>
<xsl:param name='lang' as='xs:NMTOKEN'>en</xsl:param>

<!-- Keyword used in template name='sorted-index' -->
<xsl:param name='keyword' as='xs:NMTOKEN'>indexentry</xsl:param>

<!--
Sort specified index entries (it is allowed to have several kinds of entries in the input file)
-->
<xsl:template name='sort-index-items'>
  <xsl:param name='entry' as='xs:string' required='yes' tunnel='yes'/>
  <xsl:param name='x-lang' as='xs:string' required='yes' tunnel='yes'/>
  <xsl:param name='x-case-order' as='xs:string' required='yes' tunnel='yes'/>
  <xsl:param name='items' as='item()*' select='$index-items'/>

  <xsl:for-each-group select='$items[name() = $entry]' group-by='key'>

    <!-- You may define your language or collation in the sort below. -->
    <xsl:sort select='current-grouping-key()' data-type='text'
              case-order='{$x-case-order}' lang='{$x-lang}'/>
    <xsl:variable name='key' as='xs:string' select='current-grouping-key()'/>

    <xsl:for-each-group select='current-group()' group-by='text'>

      <!-- You may define your language or collation in the sort below. -->
      <xsl:sort select='current-grouping-key()' data-type='text'
                case-order='{$x-case-order}' lang='{$x-lang}'/>
      <xsl:element name='{$entry}'>
        <xsl:attribute name='id'>
          <xsl:value-of select='$key'/>
        </xsl:attribute>
        <xsl:attribute name='text'>
          <xsl:value-of select='current-grouping-key()'/>
        </xsl:attribute>
        <xsl:call-template name='sort-index-items'>
          <xsl:with-param name='items' select='current-group()/element()[name() = $entry]'/>
        </xsl:call-template>
        <xsl:call-template name='process-page-numbers'>
          <xsl:with-param name='items' select='current-group()/page'/>
          <xsl:with-param name='keytext'
               select='if ($key = current-grouping-key()) then $key
                       else concat($key, $actual, current-grouping-key())'/>
        </xsl:call-template>
      </xsl:element>
    </xsl:for-each-group>
  </xsl:for-each-group>
</xsl:template>

<!-- Expand page range specifications, remove duplicates and collapse sequences into ranges -->
<xsl:template name='process-page-numbers'>
  <xsl:param name='entry' as='xs:string' required='yes' tunnel='yes'/>
  <xsl:param name='items' as='item()*' required='yes'/>
  <xsl:param name='keytext' as='xs:string' required='yes'/>
  <xsl:variable name='expanded-ranges' as='item()*'>
    <xsl:for-each-group select='$items[@range]' group-starting-with='*[@range = "open"]'>
      <xsl:if test='count(current-group()) &lt; 2'>
        <xsl:message>
          <xsl:value-of select=
              "concat('Warning: Lonely ',$entry,' with range=',self::*[1]/@range,
                      ' for ',$keytext,' on page ',self::*[1]/@number)"/>
        </xsl:message>
      </xsl:if>
      <xsl:if test='count(current-group()) &gt; 2'>
        <xsl:message>
          <xsl:value-of select=
              "concat('Warning: Extra ',$entry,' entries',' for ',$keytext,
              ' on page(s) ',string-join(current-group()/@number, ', '))"/>
        </xsl:message>
      </xsl:if>
      <xsl:variable name='style' as='xs:string' select='self::*[1]/@style'/>
      <xsl:for-each select='self::*[1]/@number to current-group()[last()]/@number'>
        <page number='{.}' style='{$style}'/>
      </xsl:for-each>
    </xsl:for-each-group>
  </xsl:variable>
  <xsl:variable name='sorted-pages' as='item()*'>
    <xsl:for-each select='$items[not(@range)], $expanded-ranges'>
      <xsl:sort select='@number' data-type='number'/>
      <xsl:sort select='@style'/>
      <xsl:copy-of select='.'/>
    </xsl:for-each>
  </xsl:variable>
  <xsl:variable name='collapsed-ranges' as='item()*'>
    <xsl:for-each-group select='$sorted-pages' group-adjacent='@style'>
      <xsl:variable name='style' as='xs:string' select='self::*/@style'/>
      <xsl:variable name='distinct-pages' as='xs:integer*'
                    select='distinct-values(current-group()/@number)'/>
      <xsl:variable name='pages' as='item()*'>
        <xsl:for-each select='1 to count($distinct-pages)'>
          <page number='{subsequence($distinct-pages, ., 1)}' style='{$style}'>
            <xsl:variable name='pg' as='xs:integer*' select='subsequence($distinct-pages, ., 2)'/>
            <xsl:if test='. = count($distinct-pages) or min($pg) &lt; max($pg) - 1'>
              <xsl:attribute name='range'>close</xsl:attribute>
            </xsl:if>
          </page>
        </xsl:for-each>
      </xsl:variable>
      <xsl:for-each-group select='$pages' group-ending-with='*[@range="close"]'>
        <xsl:choose>
          <xsl:when test='current-group()[last()]/@number - self::*/@number &gt; 1'>
            <page number='{self::*/@number}' rangeTo='{current-group()[last()]/@number}'
                  style='{self::*/@style}'/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:for-each select='current-group()'>
              <xsl:copy>
                <xsl:copy-of select='@* except @range'/>
              </xsl:copy>
            </xsl:for-each>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:for-each-group>
    </xsl:for-each-group>
  </xsl:variable>
  <xsl:for-each select='$collapsed-ranges'>
    <xsl:sort select='@number' data-type='number'/>
    <xsl:sort select='if (@rangeTo) then @rangeTo else -99999' data-type='number'/>
    <xsl:sort select='@style'/>
    <xsl:copy-of select='.'/>
  </xsl:for-each>
</xsl:template>

<!-- Debug sorting -->
<xsl:template name='sorted-index'>
  <xsl:param name='entry' as='xs:string' select='$keyword'/>
  <xsl:param name='x-lang' as='xs:NMTOKEN' select='$lang'/>
  <xsl:param name='x-case-order' as='xs:NMTOKEN' select='$case-order'/>
  <xsl:param name='items' as='item()*' select='$index-items'/>
  <sorted-index>
    <xsl:call-template name='sort-index-items'>
      <xsl:with-param name='entry' tunnel='yes' select='$entry'/>
      <xsl:with-param name='x-lang' tunnel='yes' select='$x-lang'/>
      <xsl:with-param name='x-case-order' tunnel='yes' select='$x-case-order'/>
      <xsl:with-param name='items' as='item()*' select='$items'/>
    </xsl:call-template>
  </sorted-index>
</xsl:template>

</xsl:stylesheet>
